/* 
 * tclAppInit.c --
 *
 *	Provides a default version of the main program and Tcl_AppInit
 *	procedure for Tcl applications (without Tk).
 *
 * Copyright (c) 1993 The Regents of the University of California.
 * Copyright (c) 1994-1997 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * SCCS: @(#) tclAppInit.c 1.20 97/03/24 14:29:43
 */

#ifdef TCL_XT_TEST
#include <X11/Intrinsic.h>
#endif

#include <tcl8.0.h>
#include <vector>
#include "solver.h"
#include <string.h>

struct solver_params_ {
	char* fs;
	double startX;
	double endX;
	double startY;
	double endY;
	int sampleX;
        int sampleY;
	int solveType;	
	int interpolation_points;
} solver_params;

char* strUsage= "Usage:\n"
"  solve command params\n"
"commands:\n"
"  setfunktional funktional\n"
"  setwindow     startX endX startY endY\n"
"  setsamples    sampleX sampleY\n"
"  settype       nr (0=brute force, 1=optimize)\n"
"  setipoints    interpolation points (odd!)\n"
"  execute       samples...\n";

int tcl_dosolve (ClientData clientData, Tcl_Interp *interp, int objc,
        Tcl_Obj *const objv[]);


extern "C" 
{
int Solve_Init(Tcl_Interp* interp)
{
    // create commands
    Tcl_CreateObjCommand(interp, "solve", tcl_dosolve, NULL, 
        (Tcl_CmdDeleteProc *) NULL);

    // provide package information
    Tcl_PkgProvide(interp, "solve", "0.1");

    solver_params.fs = NULL;
    solver_params.startX = 0;
    solver_params.endX = 1;
    solver_params.startY = 0;
    solver_params.endY = 1;
    solver_params.sampleX = 5;
    solver_params.sampleY = 13;
    solver_params.solveType = 1;
    solver_params.interpolation_points = 3;

    return TCL_OK ;
}
}

int StrSelect (char* str, char* strArray[])
{
    for (int i=0; strArray[i]; ++i) {
        if (!strcmp(str,strArray[i])) {
            // match found, return position
            return i;
        }
    }

    // not found
    return -1;
}


int tcl_dosolve (ClientData,  Tcl_Interp *interp, int objc,
        Tcl_Obj *const objv[]) 
{
	enum  COMMANDS     { SETFUNKTIONAL, SETWINDOW, SETSAMPLES, 
			     SETTYPE, SETIPOINTS, EXECUTE};
	char* Commands[] = {"setfunktional","setwindow","setsamples",
			    "settype","setipoints", "execute", NULL};

	vector<double>* samples;
	Tcl_Obj** rlist;
	Tcl_Obj* rval;
	char* err;

	if (objc < 3) {
		// return usage
		Tcl_SetResult(interp, strUsage, TCL_STATIC);
		return TCL_OK;
	}
	switch (StrSelect(Tcl_GetStringFromObj(objv[1], NULL), 
			  Commands)) {
	case SETFUNKTIONAL:		
		free(solver_params.fs);
		solver_params.fs = strdup(Tcl_GetStringFromObj(objv[2], NULL));
		break;
	case SETWINDOW:
		if (objc < 6) {
			Tcl_SetResult(interp, strUsage, TCL_STATIC);
			return TCL_ERROR;
		}
		Tcl_GetDoubleFromObj(interp, objv[2], &solver_params.startX);
		Tcl_GetDoubleFromObj(interp, objv[3], &solver_params.endX);
		Tcl_GetDoubleFromObj(interp, objv[4], &solver_params.startY);
		Tcl_GetDoubleFromObj(interp, objv[5], &solver_params.endY);
		break;
	case SETSAMPLES:
		if (objc < 4) {
			Tcl_SetResult(interp, strUsage, TCL_STATIC);
			return TCL_ERROR;
		}
		Tcl_GetIntFromObj(interp, objv[2], &solver_params.sampleX);
		Tcl_GetIntFromObj(interp, objv[3], &solver_params.sampleY);
		break;
	case SETTYPE:
		Tcl_GetIntFromObj(interp, objv[2], &solver_params.solveType);
		break;
	case SETIPOINTS:
		Tcl_GetIntFromObj(interp, objv[2], 
				  &solver_params.interpolation_points);
		break;
	case EXECUTE:
		samples = new vector<double>(solver_params.sampleX);
		if (objc-2 > solver_params.sampleX) {
			objc = solver_params.sampleX+2;
		}
		for (int i=2; i < objc; i++) {
			Tcl_GetDoubleFromObj(interp, objv[i], 
					     &(*samples)[i-2]);
		}
		err = execSolve(solver_params.fs, 
				solver_params.startX, 
				solver_params.endX, 
				solver_params.startY, 
				solver_params.endY, 
				solver_params.sampleX, 
				solver_params.sampleY, 
				*samples, 
				solver_params.solveType,
				solver_params.interpolation_points);
		if (err) {
			Tcl_SetResult(interp, err, TCL_STATIC);
			return TCL_ERROR;
		}

		rlist = new Tcl_Obj*[solver_params.sampleX];
		
		for (int i=0; i < solver_params.sampleX; i++) {
			rlist[i] = Tcl_NewDoubleObj((*samples)[i]);
		}

		delete samples;

		rval = Tcl_NewListObj(solver_params.sampleX, rlist);
		Tcl_SetObjResult(interp, rval);
		return TCL_OK;
		break;
	default:
		Tcl_SetResult(interp, strUsage, TCL_STATIC);
		return TCL_ERROR;
	}
	return TCL_OK;
}











